import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Owner } from '../../../../../models/owner.model';
import { Structure } from '../../../../../models/structure.model';
import { NotificationService } from '../../../../../services/notification.service';

@Component({
  selector: 'app-appointment-end',
  templateUrl: './appointment-end.component.html',
  styleUrls: ['./appointment-end.component.scss'],
})
export class AppointmentEndComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Input() structureRDV: Structure;
  @Input() socialWorkerName: Owner;
  @Output() setResetOrientation = new EventEmitter();
  @Output() checkValidation = new EventEmitter<any>();

  constructor(private notificationService: NotificationService) {}

  async ngOnInit(): Promise<void> {
    this.checkValidation.emit();
    try {
      if (!this.structureRDV) {
        this.setResetOrientation.emit();
      }
    } catch (error) {
      console.error(error);
      this.notificationService.showError('Une erreur est survenue', error.message);
    }
  }
}
