import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { AbstractControl, UntypedFormGroup } from '@angular/forms';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Owner } from '../../../../../models/owner.model';
import { Structure } from '../../../../../models/structure.model';
import { StructureService } from '../../../../../services/structure.service';

@Component({
  selector: 'app-make-appointment',
  templateUrl: './make-appointment.component.html',
  styleUrls: ['./make-appointment.component.scss'],
})
export class MakeAppointmentComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Input() structureRDV: Structure;
  @Output() checkValidation = new EventEmitter<any>();
  @Output() socialWorkerName = new EventEmitter<Owner>();

  public owners: Owner[];
  //public selectedOption: string;

  constructor(private structureService: StructureService) {}

  ngOnInit(): void {
    this.checkValidation.emit();

    this.findOwners(this.structureRDV._id).subscribe((owners) => {
      this.owners = owners.filter((owner) => owner.withAppointment && owner.job?.hasPersonalOffer);
    });
  }

  private findOwners(structureId: string): Observable<Owner[]> {
    return this.structureService.getStructureWithOwners(structureId).pipe(map((result) => result.owners));
  }

  public getStructureControl(nameControl: string): AbstractControl {
    return this.form.get(nameControl);
  }

  public onRadioChange(socialWorker: Owner): void {
    this.socialWorkerName.emit(socialWorker);
    this.checkValidation.emit();
  }

  public onDetailsChange(): void {
    this.checkValidation.emit();
  }
}
